
package use_trafficintersection;
import use_trafficintersection.algorithm.*;

import java.awt.*;
import javax.swing.*;

/**
 * @author Sjoerd Westendorp
 */

/*
This class is the interface between Main and RenderWorld and
also contains the JFrame object (the window).

Besides that:
- Measures FPS, which is an integer so it may twitch between values.
- Defines window size
- Moves Main.debug and Main.debugInt to RenderWorld 
- Although the bool paused is located here so Main can acces it, 
    the implementation of its features is done in RenderWorld.
*/

public class Renderer {
    // Other classes
    public static Main mainobject;
    public static Lights lights;
    
    // Public data
    public boolean paused = false;
    public boolean debugData = false;
    public int fps = 0; // frames per second
    
    // Constants
    public static final String WINDOW_TITLE = "Traffic Intersection";
    public static final int TOPBAR_ROW_SIZE = 30;
    public static final int TOPBAR_SIZE = TOPBAR_ROW_SIZE*3;
    public static final int WORLD_SIZE = 700;
    public static final int W = WORLD_SIZE;
    public static final int H = WORLD_SIZE+TOPBAR_SIZE;
    public static final int DEBUG_INT_NOSHOW = -9999999;
    public static final int CAR_MAX_WIDTH = RenderWorld.LANESIZE /2;
    
    // Constant lanes
    public static final int JUNCSIZE = 2*RenderWorld.LANESIZE + RenderWorld.SIDESIZE;
    public static final int LANE_OFFSET = RenderWorld.LANESIZE / 2 + 1 ;
    public static final int LANE_R_OFFSET = 3*RenderWorld.LANESIZE / 2 + 1 ;
    public static final int SIDE_OFFSET = 2*RenderWorld.LANESIZE + RenderWorld.SIDESIZE / 2;
    public static final int START_OFFSET = WORLD_SIZE/2 + 20;
    public static int convertToRenderX(int x){ return x+WORLD_SIZE/2; }
    public static int convertToRenderY(int y){ return y+WORLD_SIZE/2+TOPBAR_SIZE; }
    
    // Frame
    public JFrame frame;
    
    // World JPanel
    public RenderWorld renderWorld;
    
    // Stats
    public long drawCount = 0;
    public long loopCount = 0;
    
    // ------------------------------------
    
    public Renderer(Lights l){ // Constructor
	lights = l;
	
	// Refrence to this object needed later on
	Renderer renderer = this;
	
	// Start the frame rate counter
	fps_setup();
	
	// Create frame, some swing wierdness
	SwingUtilities.invokeLater(new Runnable() {
	    @Override
	    public void run() {
		// The order of the following functions is very explicit.
		// A different order creates incorrect results.
		
		// Create our window
		frame = new JFrame(WINDOW_TITLE);
		// Create the world renderer (also does action handling)
		renderWorld = new RenderWorld(renderer,lights);
		// Lets define the size of the world incl topbar
		renderWorld.setPreferredSize(new Dimension(W,H));
		
		// Add the world to the frame
		frame.add(renderWorld,BorderLayout.CENTER);
		// Dynamic sizes is too hard and unnessencary
		frame.setResizable(false);
		// Make sure the frame exactlty fits the contents
		frame.pack();
		// Pace window somehere on screen
		frame.setLocation(100, 100);
		// Ofcourse we want the window to be vsiable
		frame.setVisible(true);
		// Some orginazational stuff
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    }
	});
    }
    
    public void draw(){
	fps_tick();
	drawCount++;
	
	// Tell RenderWorld about the debug strings
	renderWorld.debugString1 = mainobject.debug;
	if( mainobject.debugInt != DEBUG_INT_NOSHOW)
	    renderWorld.debugString0 = Integer.toString(mainobject.debugInt);
	else
	    renderWorld.debugString0 = "";
	
	mainobject.draw();
    }
    
    public void loop(){
	int count = (int) renderWorld.sliderSimSpeed.state;
	try{
	    for(int i = 0; i < count; i++){
		loopCount++;
		mainobject.loop();
	    }
	}
	catch(Main.CollisionException e){
	    renderWorld.collsionPause(e.getText());
	}
    }
    
    public void reset(){
	mainobject.reset();
	Main.autoTester.reset();
    }
    
    // ------------------------------------
    //		API WITH MAIN
    // ------------------------------------
    
    public float getCars(){ return renderWorld.sliderCars.state; }
    public float getAutos(){ return renderWorld.sliderAutos.state; }
    public float getPeds(){ return renderWorld.sliderPeds.state; }
    public float[] getBias(){
	float bias[] = new float[4];
	bias[0] = renderWorld.sliderBiasU.state;
	bias[1] = renderWorld.sliderBiasD.state;
	bias[2] = renderWorld.sliderBiasL.state;
	bias[3] = renderWorld.sliderBiasR.state;
	return bias;
    }
    public float getPacketLoss(){ return renderWorld.sliderPacketLoss.state; }
    
    public void drawCross(int x, int y){
	x = convertToRenderX(x);
	y = convertToRenderY(y);
	renderWorld.drawCross(x,y,8,Color.RED);
    }
    
    public void drawRect(int x, int y, int w, int h, Color c){
	x = convertToRenderX(x);
	y = convertToRenderY(y);
	renderWorld.drawRect(x,y,w,h,c);
    }
    
    public void drawRectRotate(int x, int y, int w, int h, Color c, float rotation){
	x = convertToRenderX(x);
	y = convertToRenderY(y);
	renderWorld.drawRectRotate(x,y,w,h,c,(double) rotation);
    }
    
    public void drawCar(int x, int y, int w, int h, Color c, float direction){
	x = convertToRenderX(x);
	y = convertToRenderY(y);
	
	// Get color
	float f[] = Color.RGBtoHSB(c.getRed(),c.getGreen(),c.getBlue(),null);
	// Modify color
	Color cc = Color.getHSBColor(f[0], f[1], f[2]*0.5f);
	renderWorld.drawCar(x,y,w,h,c,cc,(double) direction);
    }
    
    public void drawPed(int x, int y, int r, Color c){
	x = convertToRenderX(x);
	y = convertToRenderY(y);
	renderWorld.drawPed(x,y,r,c);
    }
    
    // ------------------------------------
    //		FPS Measuring
    // ------------------------------------
    
    private long fps_lastTimeMillis = 0;
    private int fps_draws = 0;
    
    private void fps_setup(){
	fps_lastTimeMillis = System.currentTimeMillis();
	fps_draws = 0;
    }
    
    private void fps_tick(){
	fps_draws++;
	
	if(fps_lastTimeMillis + 1000 < System.currentTimeMillis() ){
	    fps_lastTimeMillis = System.currentTimeMillis();
	    fps = fps_draws;
	    fps_draws = 0;
	}
    }
    
    // ------------------------------------
    //		RENDERWORLD INTERACTION
    // ------------------------------------
    
    public void setTitleAddition(String title){
	frame.setTitle(WINDOW_TITLE+" "+title);
    }
    
    public void hotkey(int num){
	Main.autoTester.hotkey(num);
    }
}
